@@ -26,7 +26,7 @@
  *
  *    11/09/1997 - added chr ioctls VG_STATUS_GET_COUNT
  *                 and VG_STATUS_GET_NAMELIST
- *    01/18/1997 - change chr_open/close lock handling
+ *    01/18/1997 - change lvm_chr_open/close lock handling
  *    04/30/1998 - changed LV_STATUS ioctl to LV_STATUS_BYNAME and
  *               - added   LV_STATUS_BYINDEX ioctl
  *               - used lvm_status_byname_req_t and
@@ -35,9 +35,17 @@
  *    05/08/1998 - added support to set/clear extendable flag in volume group
  *    05/09/1998 - changed output of lvm_proc_get_info() because of
  *                 support for free (egg. longer) logical volume names
- *    05/12/1998 - added spin_locks (thanks to Pascal van Dam)
+ *    05/12/1998 - added spin_locks (thanks to Pascal van Dam
+ *                 <pascal@ramoth.xs4all.nl>)
  *    05/25/1998 - fixed handling of locked PEs in lvm_map() and lvm_chr_ioctl()
  *    05/26/1998 - reactivated verify_area by access_ok
+ *    06/07/1998 - used vmalloc/vfree instead of kmalloc/kfree to go
+ *                 beyond 128/256 KB max allocation limit per call
+ *               - #ifdef blocked spin_lock calls to avoid compile errors
+ *                 with 2.0.x
+ *    06/11/1998 - another enhancement to spinlock code in lvm_chr_open()
+ *                 and use of LVM_VERSION_CODE instead of my own macros
+ *                 (thanks to  Michael Marxmeier <mike@msede.com>)
  *
  */
 
@@ -45,7 +53,7 @@
  * TODO
  *
  *   - add code for LV specific read_ahead sectors?
- *   - implement own handling of unavailable physical volumes
+ *   - implement special handling of unavailable physical volumes
  *
  */
 
@@ -62,9 +70,17 @@
 #endif
 
 #include <linux/version.h>
-#include <linux/malloc.h>
+/* for 2.0.x series */
+#ifndef KERNEL_VERSION
+#  define KERNEL_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
+#endif
+
 #include <linux/kernel.h>
-#include <linux/mm.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION ( 2, 1 ,0)
+#  include <linux/mm.h>
+#else
+#  include <linux/vmalloc.h>
+#endif
 #include <linux/hdreg.h>
 #include <linux/stat.h>
 #include <linux/fs.h>
@@ -74,12 +90,12 @@
 #include <asm/ioctl.h>
 #include <asm/segment.h>
 
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
 #  include <asm/uaccess.h>
-#  define	memcpy_fromfs		copy_from_user
-#  define	memcpy_tofs		copy_to_user
-#  define	proc_register_dynamic	proc_register
-#  define	verify_area		!access_ok
+#  define   memcpy_fromfs     copy_from_user
+#  define   memcpy_tofs    copy_to_user
+#  define   proc_register_dynamic   proc_register
+#  define   verify_area    !access_ok
 #endif
 
 #ifdef CONFIG_KERNELD
@@ -91,7 +107,7 @@
 #include <lvm_kernel.h>
 
 #ifdef MODULE
-#   define MODULE_NAME	"lvm"
+#   define MODULE_NAME  "lvm"
 #endif
 
 
@@ -115,6 +131,7 @@ static char dummy_buf[160]; /* sized for 2 lines in lvm_proc_get_info */
 static uint lv_open = 0;
 
 static const char* const lvm_name = "lvm";
+static int lvm_reset_spindown = 0;
 static int lock = 0;
 static int loadtime = 0;
 static uint vg_count = 0;
@@ -135,22 +152,22 @@ static struct proc_dir_entry lvm_proc_entry = {
 
 static struct file_operations lvm_chr_fops = {
    NULL,                /* No lseek              */
-   NULL,		/* No read               */
-   NULL,		/* No write              */
-   NULL,		/* No readdir            */
-   NULL,		/* No select             */
+   NULL,    /* No read               */
+   NULL,    /* No write              */
+   NULL,    /* No readdir            */
+   NULL,    /* No select             */
    lvm_chr_ioctl,
-   NULL,		/* No mmap               */
+   NULL,    /* No mmap               */
    lvm_chr_open,
    lvm_chr_close,
-   NULL,		/* No fsync              */
-   NULL,		/* No fasync             */
-   NULL,		/* No check_media_change */
-   NULL		        /* No revalidate         */
+   NULL,    /* No fsync              */
+   NULL,    /* No fasync             */
+   NULL,    /* No check_media_change */
+   NULL             /* No revalidate         */
 };
 
 static struct file_operations lvm_blk_fops = {
-   NULL,           /* No lseek				 */
+   NULL,           /* No lseek             */
    block_read,     /* generic read          */
    block_write,    /* genric write          */
    NULL,           /* No readdir            */
@@ -247,8 +264,7 @@ static int lvm_init ( void)
 /*
  * Module cleanup...
  */
-void cleanup_module ( void)
-{
+void cleanup_module ( void) {
    /* DANGER: another one might be in chain beyond us */
    if ( gendisk_head != &lvm_gendisk) {
       printk ( "%s -- ERROR: another one is in chain "
@@ -322,21 +338,25 @@ static void lvm_init_vars ( void)
 
 /*
  * character device open routine
+ * Only one process can open /dev/lvm at one time, others will block.
  */
 static int lvm_chr_open ( struct inode *inode,
                           struct file *file) {
    int minor = MINOR ( inode->i_rdev);
-   ulong flags = 0;
-   spinlock_t lvm_lock_dummy, lvm_lock = SPIN_LOCK_UNLOCKED;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
+   ulong flags;
+   spinlock_t lvm_lock = SPIN_LOCK_UNLOCKED;
+   
+   /* for UP gcc will complain about unused variables */
+   ( void) flags;
+   ( void) lvm_lock;
+#endif
 
 #ifdef DEBUG
    printk ( "%s -- lvm_chr_open MINOR: %d  VG#: %d  LV#: %d  lock: %d\n",
             lvm_name, minor, VG(minor), LV(minor), lock);
 #endif
 
-   /* Just for gcc */
-   lvm_lock_dummy = lvm_lock;
-
    /* super user validation */
    if ( ! suser()) return -EACCES;
 
@@ -345,21 +365,44 @@ static int lvm_chr_open ( struct inode *inode,
 
    MOD_INC_USE_COUNT;
 
-   spin_lock_irqsave ( &lvm_lock, flags);
-   if ( lock == 0) {
-       lock = current->pid;
-   } else if ( lock != current->pid) {
-sleep_on:
-       spin_lock_irqsave ( &lvm_lock, flags);
-       while ( lock != 0) interruptible_sleep_on ( &lvm_wait);
-       spin_lock_irqsave ( &lvm_lock, flags);
-       lock = current->pid;
-       if ( lock != current->pid) goto sleep_on;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
+try_again: ;
+   spin_lock ( &lvm_lock);
+   if( lock && lock != current->pid ) {
+      printk ("lvm_chr_open: %s is locked by pid %d ...\n", lvm_name, lock);
+      spin_unlock ( &lvm_lock);
+      interruptible_sleep_on ( &lvm_wait);
+      if ( current->sigpending != 0) {
+         MOD_DEC_USE_COUNT;
+         return -EINTR;
+      }
+      if ( lvm_reset_spindown > 0) {
+         if ( GET_USE_COUNT ( &__this_module) < 2) lvm_reset_spindown = 0;
+         MOD_DEC_USE_COUNT;
+         return -EACCES;
+      }
+      goto try_again;
    }
-   spin_unlock_irqrestore( &lvm_lock, flags);
+   lock = current->pid;
+   spin_unlock (&lvm_lock);
+#else
+   while( lock && lock != current->pid ) {
+      printk ( "lvm_chr_open: %s is locked by pid %d ...\n", lvm_name, lock);
+      interruptible_sleep_on ( &lvm_wait);
+      if ( current->signal &~ current-blocked) {
+         MOD_DEC_USE_COUNT;
+         return -EINTR;
+      }
+      if ( lvm_reset_spindown > 0) {
+         if ( mod_use_count_ < 2) lvm_reset_spindown = 0;
+         MOD_DEC_USE_COUNT;
+         return -EACCES;
+      }
+   }
+   lock = current->pid;
+#endif 
 
    lvm_chr_open_count++;
-
    return 0;
 }
 
@@ -421,43 +464,45 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
                      vg[v]->vg_status &= ~VG_ACTIVE;
                      invalidate_buffers ( vg[v]->lv[l]->lv_dev);
                      if ( vg[v]->lv[l]->lv_current_pe != NULL) {
-#ifdef DEBUG_KFREE
-                        printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+                        printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-                        kfree ( vg[v]->lv[l]->lv_current_pe);
+                        vfree ( vg[v]->lv[l]->lv_current_pe);
                      }
-#ifdef DEBUG_KFREE
-                     printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+                     printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-                     kfree ( vg[v]->lv[l]);
+                     vfree ( vg[v]->lv[l]);
                   }
                }
                for ( p = 0; p < vg[v]->pv_max; p++)
                   if ( vg[v]->pv[p] != NULL) {
-#ifdef DEBUG_KFREE
-                     printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+                     printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
 #ifdef LVM_GET_INODE
                      clear_inode ( vg[v]->pv[p]->inode);
 #endif
-                     kfree ( vg[v]->pv[p]);
+                     vfree ( vg[v]->pv[p]);
                   }
-#ifdef DEBUG_KFREE
-               printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+               printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-               kfree ( vg[v]);
+               vfree ( vg[v]);
                vg[v] = NULL;
             }
          }
 #ifdef MODULE
-#if ( OS_Major > 1) && ( OS_Minor > 0)
-         (&__this_module)->usecount = lvm_chr_open_count;
+         /* does this handle a blocked open on /dev/lvm? -- mm */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
+         ( &__this_module)->usecount = lvm_chr_open_count;
 #else
          mod_use_count_ = lvm_chr_open_count;
 #endif
 #endif
          lock = 0; /* release lock */
-         wake_up ( &lvm_wait);
+         lvm_reset_spindown = 1;
+         wake_up_interruptible ( &lvm_wait);
          return 0;
 #endif /* LVM_TOTAL_RESET */
 
@@ -485,7 +530,7 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
                 sizeof(long))) != 0) return ret;
 #endif
          /* add code for read ahead array here */
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
          copy_to_user ( ( long*) arg, &read_ahead[MAJOR( inode->i_rdev)],
                         sizeof ( read_ahead[MAJOR( inode->i_rdev)]));
 #else
@@ -568,11 +613,11 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
       /* create a VGDA */
       case VG_CREATE:
          if ( vg[VG(minor)] == NULL)
-            vg[VG(minor)] = kmalloc ( sizeof ( vg_t), GFP_KERNEL);
+            vg[VG(minor)] = vmalloc ( sizeof ( vg_t));
          else return -EPERM;
 
          if ( vg[VG(minor)] == NULL) {
-            printk ( "%s -- VG_CREATE: kmalloc error VG\n", lvm_name);
+            printk ( "%s -- VG_CREATE: vmalloc error VG\n", lvm_name);
             return -ENOMEM;
          }
 
@@ -591,15 +636,15 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
             error = TRUE;
          }
          if ( vg[VG(minor)]->lv_max > ABS_MAX_LV) {
-            printk ( "%s -- Can't activate VG: ABS_MAX_LV to small for %lu\n",
+            printk ( "%s -- Can't activate VG: ABS_MAX_LV to small for %u\n",
                      lvm_name, vg[VG(minor)]->lv_max);
             error = TRUE;
          }
          if ( error == TRUE) {
-#ifdef DEBUG_KFREE
-            printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+            printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-            kfree ( vg[VG(minor)]);
+            vfree ( vg[VG(minor)]);
             vg[VG(minor)] = NULL;
             return -ENOMEM;
          }
@@ -609,9 +654,9 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
          for ( p = 0; p < vg[VG(minor)]->pv_max; p++) {
             /* user space address */
             if ( ( pvp = vg[VG(minor)]->pv[p]) != NULL) {
-               vg[VG(minor)]->pv[p] = kmalloc ( sizeof ( pv_t), GFP_KERNEL);
+               vg[VG(minor)]->pv[p] = vmalloc ( sizeof ( pv_t));
                if ( vg[VG(minor)]->pv[p] == NULL) {
-                  printk ( "%s -- VG_CREATE: kmalloc error PV\n", lvm_name);
+                  printk ( "%s -- VG_CREATE: vmalloc error PV\n", lvm_name);
                   for ( ; p < vg[VG(minor)]->pv_max; p++)
                      vg[VG(minor)]->pv[p] = NULL;
                   do_vg_remove ( minor);
@@ -639,9 +684,9 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
          for ( l = 0; l < vg[VG(minor)]->lv_max; l++) {
             /* user space address */
             if ( ( lvp = vg[VG(minor)]->lv[l]) != NULL) {
-               vg[VG(minor)]->lv[l] = kmalloc ( sizeof ( lv_t), GFP_KERNEL);
+               vg[VG(minor)]->lv[l] = vmalloc ( sizeof ( lv_t));
                if ( vg[VG(minor)]->lv[l] == NULL) {
-                  printk ( "%s -- VG_CREATE: kmalloc error LV\n", lvm_name);
+                  printk ( "%s -- VG_CREATE: vmalloc error LV\n", lvm_name);
                   do_vg_remove ( minor);
                   return -ENOMEM;
                }
@@ -667,8 +712,8 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
                if ( pep != NULL) {
                   size = vg[VG(minor)]->lv[l]->lv_allocated_le * sizeof ( pe_t);
                   if ( ( vg[VG(minor)]->lv[l]->lv_current_pe =
-                            kmalloc ( size, GFP_KERNEL)) == NULL) {
-                     printk ( "%s -- VG_CREATE: kmalloc error LV_CURRENT_PE\n",
+                            vmalloc ( size)) == NULL) {
+                     printk ( "%s -- VG_CREATE: vmalloc error LV_CURRENT_PE\n",
                               lvm_name);
                      for ( ; l < ABS_MAX_LV + 1; l++)
                         vg[VG(minor)]->lv[l] = NULL;
@@ -703,9 +748,9 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
          if ( vg[VG(minor)]->pv_cur < vg[VG(minor)]->pv_max) {
             for ( p = 0; p < vg[VG(minor)]->pv_max; p++) {
                if ( vg[VG(minor)]->pv[p] == NULL) {
-                  vg[VG(minor)]->pv[p] = kmalloc ( sizeof ( pv_t), GFP_KERNEL);
+                  vg[VG(minor)]->pv[p] = vmalloc ( sizeof ( pv_t));
                   if ( vg[VG(minor)]->pv[p] == NULL) {
-                     printk ( "%s -- VG_EXTEND: kmalloc error PV\n", lvm_name);
+                     printk ( "%s -- VG_EXTEND: vmalloc error PV\n", lvm_name);
                      return -ENOMEM;
                   }
 #ifdef VERIFY_READ
@@ -740,18 +785,18 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
 
          for ( p = 0; p < vg[VG(minor)]->pv_max; p++) {
             if ( vg[VG(minor)]->pv[p] != NULL && 
-                 lvm_strcmp ( vg[VG(minor)]->pv[p]->pv_name,pv_name) == 0) {
+                 lvm_strcmp ( vg[VG(minor)]->pv[p]->pv_name, pv_name) == 0) {
                if ( vg[VG(minor)]->pv[p]->lv_cur > 0) return -EPERM;
                vg[VG(minor)]->pe_total -= vg[VG(minor)]->pv[p]->pe_total;
                vg[VG(minor)]->pv_cur--;
                vg[VG(minor)]->pv_act--;
-#ifdef DEBUG_KFREE
-               printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+               printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
 #ifdef LVM_GET_INODE
                clear_inode ( vg[VG(minor)]->pv[p]->inode);
 #endif
-               kfree ( vg[VG(minor)]->pv[p]);
+               vfree ( vg[VG(minor)]->pv[p]);
                vg[VG(minor)]->pv[p] = NULL;
                return 0;
             }
@@ -820,7 +865,7 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
       case LV_REMOVE:
       case LV_EXTEND:
       case LV_REDUCE:
-      	 if ( vg[VG(minor)] == NULL) return -ENXIO;
+          if ( vg[VG(minor)] == NULL) return -ENXIO;
 #ifdef VERIFY_READ
          if ( ( ret = verify_area ( VERIFY_READ, arg, sizeof ( lv_name)))
               != 0) return ret;
@@ -1018,7 +1063,7 @@ static int lvm_chr_ioctl ( struct inode *inode, struct file *file,
 /*
  * character device close routine
  */
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
 static int lvm_chr_close ( struct inode *inode, struct file *file)
 #else
 static void lvm_chr_close ( struct inode *inode, struct file *file)
@@ -1036,10 +1081,10 @@ static void lvm_chr_close ( struct inode *inode, struct file *file)
    if ( lock == current->pid) lvm_chr_open_count--;
    if ( lvm_chr_open_count == 0) {
       lock = 0; /* release lock */
-      wake_up ( &lvm_wait);
+      wake_up_interruptible ( &lvm_wait);
    }
 
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
    return 0;
 #else
    return;
@@ -1129,7 +1174,7 @@ static int lvm_blk_ioctl (struct inode *inode, struct file *file,
          if ( ( ret = verify_area ( VERIFY_WRITE, ( long *) arg,
                 sizeof ( long))) != 0) return ret;
 #endif
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
          copy_to_user ( ( long*) arg, &vg[VG(minor)]->lv[LV(minor)-1]->lv_size,
                         sizeof ( vg[VG(minor)]->lv[LV(minor)-1]->lv_size));
 #else
@@ -1173,7 +1218,7 @@ static int lvm_blk_ioctl (struct inode *inode, struct file *file,
          if ( ( ret = verify_area ( VERIFY_WRITE, ( long *) arg,
                 sizeof(long))) != 0) return ret;
 #endif
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
          copy_to_user ( ( long*) arg, &read_ahead[MAJOR( inode->i_rdev)],
                         sizeof ( read_ahead[MAJOR( inode->i_rdev)]));
 #else
@@ -1195,8 +1240,8 @@ static int lvm_blk_ioctl (struct inode *inode, struct file *file,
                                     sizeof ( struct hd_geometry))) != 0)
             return ret;
 #endif
-#if ( OS_Major > 1) && ( OS_Minor > 0)
-	{
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
+   {
            int heads = 4; short sectors = 2; long start = 0;
            short cylinders = vg[VG(minor)]->lv[LV(minor)]->lv_size / 8;
 
@@ -1260,7 +1305,7 @@ static int lvm_blk_ioctl (struct inode *inode, struct file *file,
 /*
  * block device close routine
  */
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
 static int lvm_blk_close ( struct inode *inode, struct file *file)
 #else
 static void lvm_blk_close ( struct inode *inode, struct file *file)
@@ -1279,7 +1324,7 @@ static void lvm_blk_close ( struct inode *inode, struct file *file)
 
    MOD_DEC_USE_COUNT;
 
-#if ( OS_Major > 1) && ( OS_Minor > 0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1 ,0)
    return 0;
 #else
    return;
@@ -1291,7 +1336,7 @@ static void lvm_blk_close ( struct inode *inode, struct file *file)
 /*
  * Support function /proc-Filesystem
  */
-#define	LVM_PROC_BUF	( i == 0 ? dummy_buf : &buf[sz])
+#define  LVM_PROC_BUF   ( i == 0 ? dummy_buf : &buf[sz])
 
 static int lvm_proc_get_info ( char *page, char **start, off_t pos,
                                int count, int whence) {
@@ -1324,14 +1369,14 @@ static int lvm_proc_get_info ( char *page, char **start, off_t pos,
       }
    
       if ( buf != NULL) {
-#ifdef DEBUG_KFREE
-         printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+         printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-         kfree ( buf);
+         vfree ( buf);
          buf = NULL;
       }
 
-      /* 2 times: first to get size, 2nd to fill the kmalloced buffer */
+      /* 2 times: first to get size, 2nd to fill the vmalloced buffer */
       for ( i = 0; i < 2; i++) {
          sz = 0;
          sz += sprintf ( LVM_PROC_BUF, "%d VG", vg_counter);
@@ -1340,7 +1385,7 @@ static int lvm_proc_get_info ( char *page, char **start, off_t pos,
          if ( pv_counter > 1) sz += sprintf ( LVM_PROC_BUF, "s");
          sz += sprintf ( LVM_PROC_BUF, " and %d LV", lv_counter);
          if ( lv_counter > 1) sz += sprintf ( LVM_PROC_BUF, "s");
-         sz += sprintf ( LVM_PROC_BUF, "  %d read_ahead  %d malloced  ",
+         sz += sprintf ( LVM_PROC_BUF, "  %d read_ahead  %d vmalloced  ",
                          read_ahead[LVM_MAJOR],
                          vg_counter * sizeof ( vg_t) +
                          pv_counter * sizeof ( pv_t) +
@@ -1466,9 +1511,9 @@ static int lvm_proc_get_info ( char *page, char **start, off_t pos,
          }
 
          if ( buf == NULL) {
-            if ( ( buf = kmalloc ( sz + 64, GFP_KERNEL)) == NULL) {
+            if ( ( buf = vmalloc ( sz + 64)) == NULL) {
                sz = 0;
-               return sprintf ( page, "%s - kmalloc error at line %d\n",
+               return sprintf ( page, "%s - vmalloc error at line %d\n",
                                       lvm_name, __LINE__);
             }
          }
@@ -1477,16 +1522,16 @@ static int lvm_proc_get_info ( char *page, char **start, off_t pos,
    }
 
    if ( pos > sz - 1) {
-      kfree ( buf);
+      vfree ( buf);
       buf = NULL;
-      return 0;
+      return -1;
    }
 
    *start = &buf[pos];
    if ( sz - pos < count) return sz - pos;
    else                   return count;
 }
-#endif /* #if defined LVM_CONFIG_PROC_FS && defined CONFIG_PROC_FS*/ 
+#endif /* #if defined LVM_CONFIG_PROC_FS && defined CONFIG_PROC_FS */
 
 
 /*
@@ -1578,17 +1623,18 @@ lvm_second_remap:
       *rdev = vg[VG(minor)]->lv[LV(minor)-1]->lv_current_pe[index].dev;
    }
 
+/*****************************************************************************
    if ( MAJOR ( *rdev) != SCSI_DISK_MAJOR &&
         MAJOR ( *rdev) != IDE0_MAJOR &&
         MAJOR ( *rdev) != IDE1_MAJOR &&
         MAJOR ( *rdev) != IDE2_MAJOR &&
         MAJOR ( *rdev) != IDE3_MAJOR &&
-#if ( OS_Major > 1) && ( OS_Minor > 0) && ( OS_SubMinor > 97)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION ( 2, 1, 97)
         MAJOR ( *rdev) != IDE4_MAJOR &&
         MAJOR ( *rdev) != IDE5_MAJOR &&
 #endif
         MAJOR ( *rdev) != MD_MAJOR) {
-      printk ( "lv_current_pe[%ld].pe ERROR: %ld  rdev: %02d:%02d  "
+      printk ( "lv_current_pe[%ld].pe ERROR: %u  rdev: %02d:%02d  "
                "rsector: %ld\nstripe_length: %ld  stripe_index: %ld  "
                "rsector_sav: %ld  index_offset: %ld\n",
                index,
@@ -1602,6 +1648,7 @@ lvm_second_remap:
                index_offset);
       return -1;
    }
+*****************************************************************************/
 
 #ifdef DEBUG_MAP
    printk ( "lv_current_pe[%ld].pe: %ld  rdev: %02d:%02d  rsector:%ld\n"
@@ -1658,13 +1705,13 @@ static int do_vg_remove ( int minor) {
    /* Free PVs */
    for ( p = 0; p < vg[VG(minor)]->pv_max; p++)
       if ( vg[VG(minor)]->pv[p] != NULL) {
-#ifdef DEBUG_KFREE
-         printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+         printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
 #ifdef LVM_GET_INODE
          clear_inode ( vg[VG(minor)]->pv[p]->inode);
 #endif
-         kfree ( vg[VG(minor)]->pv[p]);
+         vfree ( vg[VG(minor)]->pv[p]);
       }
 
    /* Free LVs */
@@ -1687,23 +1734,23 @@ static int do_vg_remove ( int minor) {
 
          /* Free PEs */
          if ( vg[VG(minor)]->lv[l]->lv_current_pe != NULL) {
-#ifdef DEBUG_KFREE
-            printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+            printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-            kfree ( vg[VG(minor)]->lv[l]->lv_current_pe);
+            vfree ( vg[VG(minor)]->lv[l]->lv_current_pe);
          }
          /* Free LVs */
-#ifdef DEBUG_KFREE
-         printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+         printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-         kfree ( vg[VG(minor)]->lv[l]);
+         vfree ( vg[VG(minor)]->lv[l]);
       }
    }
 
-#ifdef DEBUG_KFREE
-   printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+   printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-   kfree ( vg[VG(minor)]);
+   vfree ( vg[VG(minor)]);
    vg[VG(minor)] = NULL;
    vg_count--;
 
@@ -1730,14 +1777,14 @@ static int do_lv_create ( int minor, char *lv_name, lv_t *lv,
       if ( vg[VG(minor)]->lv[l] == NULL) {
          if ( l_new == -1) l_new = l;
       } else if ( lvm_strcmp ( vg[VG(minor)]->lv[l]->lv_name, lv_name) == 0)
-                return -EPERM;
+                return -EEXIST;
    }
    if ( l_new == -1) return -EPERM;
    l = l_new;
 
-   vg[VG(minor)]->lv[l] = kmalloc ( sizeof ( lv_t), GFP_KERNEL);
+   vg[VG(minor)]->lv[l] = vmalloc ( sizeof ( lv_t));
    if ( vg[VG(minor)]->lv[l] == NULL) {
-      printk ( "%s -- LV_CREATE: kmalloc error LV\n", lvm_name);
+      printk ( "%s -- LV_CREATE: vmalloc error LV\n", lvm_name);
       return -ENOMEM;
    }
 
@@ -1749,20 +1796,20 @@ static int do_lv_create ( int minor, char *lv_name, lv_t *lv,
    /* get the PE structures from user space */
    size = vg[VG(minor)]->lv[l]->lv_allocated_le * sizeof ( pe_t);
    if ( ( vg[VG(minor)]->lv[l]->lv_current_pe =
-          kmalloc ( size, GFP_KERNEL)) == NULL) {
-      printk ( "%s -- LV_CREATE: kmalloc error LV_CURRENT_PE of %d Byte\n",
+          vmalloc ( size)) == NULL) {
+      printk ( "%s -- LV_CREATE: vmalloc error LV_CURRENT_PE of %d Byte\n",
                lvm_name, size);
-#ifdef DEBUG_KFREE
-      printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+      printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-      kfree ( vg[VG(minor)]->lv[l]);
+      vfree ( vg[VG(minor)]->lv[l]);
       vg[VG(minor)]->lv[l] = NULL;
       return -ENOMEM;
    }
 
 #ifdef VERIFY_READ
    if ( ( ret = verify_area ( VERIFY_READ, pep, size)) != 0) {
-      kfree ( vg[VG(minor)]->lv[l]);
+      vfree ( vg[VG(minor)]->lv[l]);
       vg[VG(minor)]->lv[l] = NULL;
       return ret;
    }
@@ -1825,15 +1872,15 @@ static int do_lv_remove ( int minor, char *lv_name) {
             }
          }
 
-#ifdef DEBUG_KFREE
-         printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+         printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-         kfree ( vg[VG(minor)]->lv[l]->lv_current_pe);
+         vfree ( vg[VG(minor)]->lv[l]->lv_current_pe);
 
-#ifdef DEBUG_KFREE
-         printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+         printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-         kfree ( vg[VG(minor)]->lv[l]);
+         vfree ( vg[VG(minor)]->lv[l]);
 
          vg[VG(minor)]->lv[l] = NULL;
          vg[VG(minor)]->lv_cur--;
@@ -1866,8 +1913,8 @@ static int do_lv_extend_reduce ( int minor, char *lv_name, lv_t *lv,
    if ( l == vg[VG(minor)]->lv_max) return -ENXIO;
 
    size = lv->lv_current_le * sizeof ( pe_t);
-   if ( ( pe = kmalloc ( size, GFP_KERNEL)) == NULL) {
-      printk ( "%s -- do_lv_extend_reduce: kmalloc error LV_CURRENT_PE "
+   if ( ( pe = vmalloc ( size)) == NULL) {
+      printk ( "%s -- do_lv_extend_reduce: vmalloc error LV_CURRENT_PE "
                "of %d Byte\n", lvm_name, size);
       return -ENOMEM;
    }
@@ -1875,7 +1922,7 @@ static int do_lv_extend_reduce ( int minor, char *lv_name, lv_t *lv,
 #ifdef VERIFY_READ
    /* verify size of PE structures in user space */
    if ( ( ret = verify_area ( VERIFY_READ, pep, size)) != 0) {
-      kfree ( pe);
+      vfree ( pe);
       return ret;
    }
 #endif
@@ -1902,10 +1949,10 @@ static int do_lv_extend_reduce ( int minor, char *lv_name, lv_t *lv,
       }
    }
 
-#ifdef DEBUG_KFREE
-   printk ( "%s -- kfree %d\n", lvm_name, __LINE__);
+#ifdef DEBUG_VFREE
+   printk ( "%s -- vfree %d\n", lvm_name, __LINE__);
 #endif
-   kfree ( vg[VG(minor)]->lv[l]->lv_current_pe);
+   vfree ( vg[VG(minor)]->lv[l]->lv_current_pe);
 
    lv_open = vg[VG(minor)]->lv[l]->lv_open;
 
