@@ -2,6 +2,7 @@
  * Author: Heinz Mauelshagen, Germany
  *
  * May 1997
+ * June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,10 +21,19 @@
  *
  */
 
+/*
+ *
+ * Changelog
+ *
+ *    06/12/1998 - added debug return output
+ *
+ */
+
 #include <liblvm.h>
 
 char *lv_change_vgname ( char *vg_name, char *lv_name) {
    char *lv_name_ptr = NULL;
+   char *ret = NULL;
    static char lv_name_buf[NAME_LEN] = { 0, };
 
 #ifdef DEBUG
@@ -31,14 +41,17 @@ char *lv_change_vgname ( char *vg_name, char *lv_name) {
 #endif
 
    if ( vg_name == NULL || vg_check_name ( vg_name) < 0 ||
-        lv_name == NULL || lv_check_name ( lv_name) < 0) return NULL;
-
-   lv_name_ptr = strrchr ( lv_name, '/');
-   lv_name_ptr++;
-   sprintf ( lv_name_buf, "/dev/%s/%s%c", vg_name, lv_name_ptr, 0);
+        lv_name == NULL || lv_check_name ( lv_name) < 0) {
+      ret = NULL;
+   } else {
+      lv_name_ptr = strrchr ( lv_name, '/');
+      lv_name_ptr++;
+      sprintf ( lv_name_buf, "/dev/%s/%s%c", vg_name, lv_name_ptr, 0);
+      ret = lv_name_buf;
+   }
 
 #ifdef DEBUG
-   debug ( "lv_change_vgname -- LEAVING\n");
+   debug ( "lv_change_vgname -- LEAVING with ret: %s\n", ret);
 #endif
-   return lv_name_buf;
+   return ret;
 }
