@@ -25,6 +25,7 @@
  * Changelog
  *
  *    04/30/1998 -- change to lv_status_byname()
+ *    06/12/1998 -- added debug return output
  *
  */
 
@@ -39,15 +40,17 @@ int lv_check_active ( char *vg_name, char *lv_name) {
    debug ( "lv_check_active -- CALLED\n");
 #endif
 
-   if ( vg_name == NULL || lv_name == NULL || lv_check_name ( lv_name) < 0)
-      return -LVM_EPARA;
-
-   if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) return ret;
-   if ( lv->lv_status & LV_ACTIVE) ret = TRUE;
-   else                            ret = FALSE;
+   if ( vg_name == NULL || lv_name == NULL || lv_check_name ( lv_name) < 0) {
+      ret = -LVM_EPARAM;
+   } else {
+      if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) == 0) {
+         if ( lv->lv_status & LV_ACTIVE) ret = TRUE;
+         else                            ret = FALSE;
+      }
+   }
 
 #ifdef DEBUG
-   debug ( "lv_check_active -- LEAVING\n");
+   debug ( "lv_check_active -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
