@@ -38,7 +38,7 @@ int lv_check_exist ( char *lv_name) {
    debug ( "lv_check_exist -- CALLED\n");
 #endif
 
-   if ( lv_name == NULL || lv_check_name ( lv_name) < 0) return -LVM_EPARA;
+   if ( lv_name == NULL || lv_check_name ( lv_name) < 0) return -LVM_EPARAM;
 
    if ( ( ret = vg_read_with_pv_and_lv ( vg_name_of_lv ( lv_name), &vg)) == 0) {
       ret = FALSE;
