@@ -2,6 +2,7 @@
  * Author: Heinz Mauelshagen, Germany
  *
  * April 1997
+ * June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,23 +21,37 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    06/12/1998 - reworked return output
+ *
+ */
+
 #include <liblvm.h>
 
 int lv_check_on_pv ( pv_t *pv, int lv_num) {
    int pe;
+   int ret = FALSE;
 
 #ifdef DEBUG
    debug ( "lv_check_on_pv -- CALLED\n");
 #endif
 
-   if ( pv == NULL || lv_num < 1 || pv_check_consistency < 0) return -LVM_EPARA;
+   if ( pv == NULL || lv_num < 1 || pv_check_consistency ( pv) < 0) {
+      ret = -LVM_EPARAM;
+      goto lv_check_on_pv_end;
+   }
 
-   for ( pe = 0; pe < pv->pe_total; pe++)
-      if ( pv->pe[pe].lv_num == lv_num) return TRUE;
+   for ( pe = 0; pe < pv->pe_total; pe++) {
+      if ( pv->pe[pe].lv_num == lv_num) ret = TRUE;
+      break;
+   }
 
+lv_check_on_pv_end:
 #ifdef DEBUG
-   debug ( "lv_check_on_pv -- LEAVING\n");
+   debug ( "lv_check_on_pv -- LEAVING with ret: %d\n", ret);
 #endif
 
-   return FALSE;
+   return ret;
 }
