@@ -33,7 +33,7 @@ int lv_count_pe ( pv_t *pv, int lv_num) {
 
    if ( pv == NULL ||
         pv_check_name ( pv->pv_name) < 0 ||
-        lv_num < 1) return -LVM_EPARA;
+        lv_num < 1) return -LVM_EPARAM;
 
    for ( pe = 0; pe < pv->pe_total; pe++) 
       if ( pv->pe[pe].lv_num == lv_num) pe_count++;
