@@ -33,7 +33,7 @@ int lv_get_le_on_pv ( pv_t *pv, int lv_number) {
 #endif
 
    if ( pv == NULL || pv_check_consistency ( pv) < 0 ||
-        lv_number < 1) return -LVM_EPARA;
+        lv_number < 1) return -LVM_EPARAM;
 
    ret = 0;
    for ( pe = 0; pe < pv->pe_total; pe++) {
