@@ -32,7 +32,7 @@ int lv_le_remap ( vg_t *vg, le_remap_req_t *le_remap_req) {
 #endif
 
    if ( vg == NULL || vg_check_name ( vg->vg_name) < 0 ||
-        le_remap_req == NULL) return -LVM_EPARA;
+        le_remap_req == NULL) return -LVM_EPARAM;
 
    sprintf ( group_file, "/dev/%s/group%c", vg->vg_name, 0);
 
