@@ -29,7 +29,7 @@ int lv_number_from_name_in_vg ( char *lv_name, vg_t *vg) {
    debug ( "lv_number_from_name_in_vg -- CALLED\n");
 #endif
 
-   if ( lv_name == NULL || vg == NULL) return -LVM_EPARA;
+   if ( lv_name == NULL || vg == NULL) return -LVM_EPARAM;
 
    for ( ; l < vg->lv_max; l++) {
       if ( vg->lv[l] == NULL) continue;
