@@ -2,6 +2,7 @@
  * Author: Heinz Mauelshagen, Germany
  *
  * May 1997
+ * June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,10 +21,16 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *    06/14/1998 -- used pv_get_index_by_name() instead of own loop
+ *
+ */
+
 #include <liblvm.h>
 
 int pv_check_in_vg ( vg_t *vg, char *pv_name) {
-   int p = 0;
    int ret = FALSE;
 
 #ifdef DEBUG
@@ -32,15 +39,10 @@ int pv_check_in_vg ( vg_t *vg, char *pv_name) {
 
    if ( vg == NULL || pv_name == NULL ||
         vg_check_name ( vg->vg_name) < 0 ||
-        pv_check_name ( pv_name) < 0) return -LVM_EPARA;
-
-   for ( p = 0; p < vg->pv_cur; p++) {
-      if ( strcmp ( vg->pv[p]->pv_name, pv_name) == 0) {
-         ret = TRUE;
-         break;
-      }
-   }
+        pv_check_name ( pv_name) < 0) return -LVM_EPARAM;
 
+   if ( pv_get_index_by_name ( vg, pv_name) > -1 ) ret = TRUE;
+   
 #ifdef DEBUG
    debug ( "pv_check_in_vg -- LEAVING with ret: %d\n", ret);
 #endif
