@@ -37,7 +37,7 @@ int pv_check_new ( pv_t *pv) {
    debug ( "pv_check_new -- CALLED\n");
 #endif
 
-   if ( pv == NULL) ret = -LVM_EPARA;
+   if ( pv == NULL) ret = -LVM_EPARAM;
    else if ( pv->vg_name[0] == 0 &&
              pv->pe_total == 0) ret = TRUE;
 
