@@ -37,7 +37,7 @@ int pv_check_part ( char *dev_name) {
    debug ( "pv_check_part -- CALLED with %s\n", dev_name);
 #endif
 
-   if ( dev_name == NULL || pv_check_name ( dev_name) < 0) ret = -LVM_EPARA;
+   if ( dev_name == NULL || pv_check_name ( dev_name) < 0) ret = -LVM_EPARAM;
    else {
       ret = atoi ( dev_name + DISK_NAME_LEN);
       if ( ret < MIN_PART || ret > MAX_PART) ret = -LVM_EPV_CHECK_PART;
