@@ -32,7 +32,7 @@ int pv_check_volume ( char *dev_name, pv_t *pv) {
 
    if ( dev_name == NULL ||
         pv_check_name ( dev_name) < 0 ||
-        pv == NULL) return -LVM_EPARA;
+        pv == NULL) return -LVM_EPARAM;
 
    if ( /* strcmp  ( dev_name, pv->pv_name) == 0 && */
         ( pv_check_new ( pv) == TRUE ||
