@@ -31,7 +31,7 @@ int pv_flush ( char *pv_name) {
    debug ( "pv_flush -- CALLED to flush %s\n", pv_name);
 #endif
 
-   if ( pv_name == NULL || pv_check_name ( pv_name) < 0) return -LVM_EPARA;
+   if ( pv_name == NULL || pv_check_name ( pv_name) < 0) return -LVM_EPARAM;
 
    strcpy ( req.pv_name, pv_name);
    if ( ( lvm = open ( LVM_DEV, O_RDONLY)) == -1)
