@@ -52,12 +52,11 @@
 void pv_move_interrupt ( int);
 static int pv_move_int = FALSE;
 static lv_t *lv_this = NULL;
-static char *cmd_this = NULL;
 
 /* perform the move of all physical extends / those of a logical volume */
 int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
                   int src_pv_index, int lv_index,
-                  char *cmd, int opt_v, int opt_t) {
+                  int opt_v, int opt_t) {
    int i = 0;
    int p = 0;
    int pa = 0;
@@ -85,9 +84,8 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
         buffer == NULL || pv_allowed == NULL ||
         src_pv_index < 0 || src_pv_index >= vg->pv_cur ||
         lv_index < -1 || lv_index >= ( int) vg->lv_max ||
-        cmd == NULL || opt_v < 0 || opt_t < 0) return -LVM_EPARA;
+        opt_v < 0 || opt_t < 0) return -LVM_EPARAM;
 
-   cmd_this = cmd;
    pv_move_int = FALSE;
 
    /* space check */
@@ -210,7 +208,7 @@ int pv_move_pes ( vg_t *vg, char *buffer, char **pv_allowed,
       if ( ( ret = pv_move_pe ( vg, buffer,
                                 pe_src[pe].p,  pe_dest[pe].p,
                                 pe_src[pe].pe, pe_dest[pe].pe,
-                                cmd, opt_v, opt_t, pe + 1, pe_to_move)) < 0) {
+                                opt_v, opt_t, pe + 1, pe_to_move)) < 0) {
          fprintf ( stderr, "%s -- ERROR %d pv_move_pe\n\n", cmd, ret);
          goto pv_move_pes_end;
       }
@@ -234,7 +232,7 @@ pv_move_pes_end:
 int pv_move_pe ( vg_t *vg, char *buffer,
                  long src_pv_index, long dst_pv_index,
                  long pe_source,    long pe_dest,
-                 char *cmd, int opt_v, int opt_t,
+                 int opt_v, int opt_t,
                  int act_pe, int off_pe) {
    int in = -1;
    int out = -1;
@@ -250,8 +248,8 @@ int pv_move_pe ( vg_t *vg, char *buffer,
         src_pv_index == dst_pv_index ||
         pe_source < 0 || pe_source >= vg->pv[src_pv_index]->pe_total ||
         pe_dest   < 0 || pe_dest   >= vg->pv[dst_pv_index]->pe_total ||
-        cmd == NULL || opt_v < 0 || opt_t < 0 || act_pe < 0 || off_pe < 0 ||
-        vg->pv[dst_pv_index]->pe[pe_dest].lv_num != 0) return -LVM_EPARA;
+        opt_v < 0 || opt_t < 0 || act_pe < 0 || off_pe < 0 ||
+        vg->pv[dst_pv_index]->pe[pe_dest].lv_num != 0) return -LVM_EPARAM;
 
    if ( ( in = open ( vg->pv[src_pv_index]->pv_name, O_RDONLY)) == -1) {
       fprintf ( stderr, "%s -- couldn't open input physical volume %s\n\n",
@@ -298,7 +296,7 @@ int pv_move_pe ( vg_t *vg, char *buffer,
 
    if ( opt_v > 1) {
       printf ( "lv: %s[%d]  old_dev: %02d:%02d  new_dev: %02d:%02d  "
-               "old_pe_sector: %lu  new_pe_sector: %lu\n",
+               "old_pe_sector: %lu  new_pe_sector: %u\n",
                le_remap_req.lv_name,
                vg->pv[dst_pv_index]->pe[pe_dest].lv_num,
                MAJOR ( le_remap_req.old_dev),
@@ -396,6 +394,8 @@ int pv_move_pe ( vg_t *vg, char *buffer,
       }
    }
 
+   if ( opt_v > 1) printf ( "%s -- writing output physical volume %s\n",
+                            cmd, vg->pv[dst_pv_index]->pv_name);
    if ( opt_t == 0) {
       if ( write ( out, buffer, size) != size) {
          fprintf ( stderr, "%s -- ERROR writing output "
@@ -499,11 +499,11 @@ void pv_move_interrupt ( int sig) {
       if ( lv_this->lv_stripes > 1) {
          printf ( "%s -- interrupt of a striped logical volume move "
                   "not possible\n",
-                  cmd_this);
+                  cmd);
          return;
       } else {
          printf ( "%s -- interrupting move... Please wait.\n",
-                  cmd_this);
+                  cmd);
          pv_move_int = TRUE;
          return;
       }
