@@ -60,7 +60,7 @@ int pv_read ( char *dev_name, pv_t **pv, int *open_errno) {
 
    if ( dev_name == NULL ||
         pv == NULL ||
-        pv_check_name ( dev_name) < 0) return -LVM_EPARA;
+        pv_check_name ( dev_name) < 0) return -LVM_EPARAM;
 
    /* enhance me: flush PV in each and every case ??? */
    if ( pv_flush ( dev_name) < 0) return -LVM_EPV_READ_PV_FLUSH;
@@ -134,7 +134,7 @@ int pv_read_all_pv ( pv_t ***pv, int reread) {
 
    if ( pv == NULL ||
         ( reread != TRUE && reread != FALSE)) {
-      ret = -LVM_EPARA;
+      ret = -LVM_EPARAM;
       goto pv_read_all_pv_end;
    }
 
@@ -272,7 +272,7 @@ int pv_read_all_pv_of_vg ( char *vg_name, pv_t ***pv, int reread) {
 
    if ( pv == NULL || vg_name == NULL ||
         ( reread != TRUE && reread != FALSE) ||
-        vg_check_name ( vg_name) < 0) return -LVM_EPARA;
+        vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
 
    *pv = NULL;
 
@@ -362,7 +362,7 @@ int pv_read_pe ( pv_t *pv, disk_pe_t **pe) {
 #endif
 
    if ( pv == NULL || pe == NULL ||
-        pv_check_name ( pv->pv_name) < 0) return -LVM_EPARA;
+        pv_check_name ( pv->pv_name) < 0) return -LVM_EPARAM;
 
    *pe = NULL;
 
@@ -416,7 +416,7 @@ int pv_read_all_pe_of_vg ( char *vg_name, disk_pe_t ***pe, int reread) {
    if ( vg_name == NULL ||
         vg_check_name ( vg_name) < 0 ||
         pe == NULL ||
-        ( reread != TRUE && reread != FALSE)) return -LVM_EPARA;
+        ( reread != TRUE && reread != FALSE)) return -LVM_EPARAM;
 
    *pe = NULL;
 
@@ -445,13 +445,13 @@ int pv_read_all_pe_of_vg ( char *vg_name, disk_pe_t ***pe, int reread) {
       if ( ( pe_this = malloc ( ( pv_count+1) * sizeof ( disk_pe_t*))) == NULL){
          fprintf ( stderr, "malloc error in %s [line %d]\n",
                            __FILE__, __LINE__);
-         return -LVM_PV_READ_ALL_PE_OF_VG_MALLOC;
+         return -LVM_EPV_READ_ALL_PE_OF_VG_MALLOC;
       }
       if ( ( pe_this_sort =
              malloc ( ( pv_count+1) * sizeof ( disk_pe_t*))) == NULL) {
          fprintf ( stderr, "malloc error in %s [line %d]\n",
                            __FILE__, __LINE__);
-         return -LVM_PV_READ_ALL_PE_OF_VG_MALLOC;
+         return -LVM_EPV_READ_ALL_PE_OF_VG_MALLOC;
       }
       for ( p = 0; pv_tmp[p] != NULL; p++) {
          if ( ( ret = pv_read_pe ( pv_tmp[p],
@@ -475,7 +475,7 @@ int pv_read_all_pe_of_vg ( char *vg_name, disk_pe_t ***pe, int reread) {
       if ( opt_d > 0) {
          printf ( "pv_read_all_pe_of_vg -- AFTER LOOP of pv_read_pe\n");
          for ( p = 0; pe_this[p] != NULL; p++)
-            printf ( "pv_read_all_pe_of_vg -- %s with %lu PE at address %X "
+            printf ( "pv_read_all_pe_of_vg -- %s with %u PE at address %X "
                      "for PV #%d\n",
                      pv_tmp[p]->pv_name,
                      pv_tmp[p]->pe_total,
@@ -514,7 +514,7 @@ int pv_read_namelist ( char *pv_name, vg_t *vg, char **pv_namelist) {
    if ( pv_name == NULL || pv_check_name ( pv_name) < 0 ||
         vg == NULL || vg_check_name ( vg->vg_name) < 0 ||
         vg->pv[0] == NULL || vg->pv[0]->pv_namelist_on_disk.base == 0 ||
-        pv_namelist == NULL) return -LVM_EPARA;
+        pv_namelist == NULL) return -LVM_EPARAM;
 
    if ( ( pv_handle = open ( pv_name, O_RDONLY)) == -1)
       ret = -LVM_EPV_READ_NAMELIST_OPEN;
