@@ -2,7 +2,7 @@
  * Author: Heinz Mauelshagen, Germany
  *
  * March-May 1997
- * January 1998
+ * January,May,June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,22 +27,27 @@
  *    01/11/1998 - seperated from previous vg_name.c
  *    02/12/1998 - changed for free volume group names
  *    05/12/1998 - avoided checking vg_name against LVM_PV_NEW
+ *    05/27/1998 - checking vg_name against EXPORTED
+ *    06/06/1998 - checking vg_name against EXPORTED was an error
+ *                 for vgexport
  *
  */
 
 #include <liblvm.h>
 
 int vg_check_name ( char *vg_name) {
+   int len = 0;
    int ret = 0;
 
 #ifdef DEBUG
    debug ( "vg_check_name -- CALLED\n");
 #endif
 
-   if ( vg_name == NULL) return -LVM_EPARA;
+   if ( vg_name == NULL) return -LVM_EPARAM;
 
    if ( strncmp ( vg_name, "/dev/", 5) == 0) strcpy ( vg_name, &vg_name[5]);
 
+   len = strlen ( EXPORTED);
    if ( strchr ( vg_name, '/') != NULL ||
         strlen ( vg_name) > ( NAME_LEN - 1) / 2) ret = -LVM_EVG_CHECK_NAME;
 
