@@ -29,7 +29,7 @@ int vg_remove_dir_and_group_and_nodes ( char *vg_name) {
    debug ( "vg_remove_dir_and_group_and_nodes -- CALLED\n");
 #endif
 
-   if ( vg_name == NULL || vg_check_name ( vg_name) < 0) return -LVM_EPARA;
+   if ( vg_name == NULL || vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
 
    /* remove the volume group directory */
    sprintf ( system_buffer, "rm -fr /dev/%s%c", vg_name, 0);
