@@ -2,7 +2,7 @@
  * Author: Heinz Mauelshagen, Germany
  *
  * March,October,November 1997
- * January,May 1998
+ * January,May,June 1998
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,10 @@
  *    04/29/1998 - changed to new lv_create_node()
  *    05/16/1998 - added lvmtab checking
  *    05/26/1998 - added message in case of low space
+ *    06/12/1998 - enhanced checking numbers in option arguments
+ *    06/14/1998 - supported physical volumes on command line for
+ *                 striped logical volumes
+ *    06/27/1998 - changed lvm_tab_* calling convention
  *
  */
 
@@ -51,6 +55,8 @@
 
 #include <lvm_user.h>
 
+char *cmd = NULL;
+
 #ifdef DEBUG
 int opt_d = 0;
 #endif
@@ -86,7 +92,6 @@ int main ( int argc, char **argv) {
 #else
    char *options = "A:C:h?i:I:l:L:n:p:vZ:";
 #endif
-   char *cmd = NULL;
    char *dummy = NULL;
    char *vg_name = NULL;
    char **pv_allowed = NULL;
@@ -143,7 +148,7 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical volume create\n\n"
+            printf ( "\n%s\n\n%s -- Logical Volume Create\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
@@ -170,9 +175,15 @@ int main ( int argc, char **argv) {
                fprintf ( stderr, "%s -- i option yet given\n\n", cmd);
                return 1;
             }
-            stripes = atoi ( optarg);
-            stripesize = LVM_DEFAULT_STRIPE_SIZE;
-            opt_i++;
+            if ( lvm_check_number ( optarg) == TRUE) {
+               stripes = atoi ( optarg);
+               stripesize = LVM_DEFAULT_STRIPE_SIZE;
+               opt_i++;
+            } else {
+               fprintf ( stderr, "%s -- ERROR option i argument \"%s\"\n",
+                                 cmd, optarg);
+               return 1;
+            }
             break;
 
          case 'I':
@@ -180,8 +191,15 @@ int main ( int argc, char **argv) {
                fprintf ( stderr, "%s -- I option yet given\n\n", cmd);
                return 1;
             }
-            stripesize = atoi ( optarg);
-            opt_I++;
+
+            if ( lvm_check_number ( optarg) == TRUE) {
+               stripesize = atoi ( optarg);
+               opt_I++;
+            } else {
+               fprintf ( stderr, "%s -- ERROR option I argument \"%s\"\n",
+                                 cmd, optarg);
+               return 1;
+            }
             break;
 
          case 'l':
@@ -193,8 +211,15 @@ int main ( int argc, char **argv) {
                fprintf ( stderr, "%s -- l option yet given\n\n", cmd);
                return 1;
             }
-            size = atoi ( optarg);
-            opt_l++;
+
+            if ( lvm_check_number ( optarg) == TRUE) {
+               size = atoi ( optarg);
+               opt_l++;
+            } else {
+               fprintf ( stderr, "%s -- ERROR option l argument \"%s\"\n\n",
+                                 cmd, optarg);
+               return 1;
+            }
             break;
 
          case 'L':
@@ -206,8 +231,15 @@ int main ( int argc, char **argv) {
                fprintf ( stderr, "%s -- L option yet given\n\n", cmd);
                return 1;
             }
-            size = atoi ( optarg);
-            opt_L++;
+
+            if ( lvm_check_number ( optarg) == TRUE) {
+               size = atoi ( optarg);
+               opt_L++;
+            } else {
+               fprintf ( stderr, "%s -- ERROR option L argument \"%s\"\n\n",
+                                 cmd, optarg);
+               return 1;
+            }
             break;
 
          case 'n':
@@ -288,10 +320,10 @@ int main ( int argc, char **argv) {
                            cmd, lv_name);
          return 1;
       } else if ( lstat ( lv_path, &stat_buf) != -1) {
-         lvm_filetype_show ( stat_buf.st_mode, cmd, lv_path);
+         lvm_show_filetype ( stat_buf.st_mode, lv_path);
          return 1;
       }
-      else if ( lv_check_exist ( lv_path) == TRUE) {
+      else if ( lvm_tab_lv_check_exist ( lv_path) == TRUE) {
          fprintf ( stderr, "%s -- logical volume %s allready exists\n\n",
                            cmd, lv_path);
          return 1;
@@ -314,9 +346,9 @@ int main ( int argc, char **argv) {
    if ( opt_i) {
       if ( opt_I == 0) printf ( "%s -- INFO: using default stripe "
                                 "size %lu KB\n", cmd,  LVM_DEFAULT_STRIPE_SIZE);
-      if ( pv_allowed != NULL) {
-         fprintf ( stderr, "%s -- no striped logical volumes with physical "
-                           "volumes on command line supported\n\n", cmd);
+      if ( pv_allowed != NULL && argc - optind != stripes) {
+         fprintf ( stderr, "%s -- invalid number of physical "
+                           "volumes on command line\n\n", cmd);
          return 1;
       }
       if ( stripes == 1) {
@@ -328,8 +360,9 @@ int main ( int argc, char **argv) {
 
    if ( opt_v > 0) printf ( "%s -- checking stripe count\n", cmd);
    if ( stripes < 1 || stripes > LVM_MAX_STRIPES) {
-      fprintf ( stderr, "%s -- invalid number of stripes: %d\n\n",
-                cmd, stripes);
+      fprintf ( stderr, "%s -- invalid number of stripes: %d\n"
+                        "%s -- must be between %d and %d\n\n",
+                        cmd, stripes, cmd, 1, LVM_MAX_STRIPES);
       return 1;
    }
 
@@ -379,7 +412,7 @@ int main ( int argc, char **argv) {
                             "volume group physical extend size\n", cmd);
    if ( stripesize > vg->pe_size) {
       fprintf ( stderr, "%s -- setting stripe size %d KB to "
-                        "physical extend size %lu KB\n",
+                        "physical extend size %u KB\n",
                         cmd, stripesize / 2, vg->pe_size / 2);
       stripesize = vg->pe_size;
    }
@@ -390,7 +423,7 @@ int main ( int argc, char **argv) {
       if ( size % vg->pe_size > 0) {
          size += vg->pe_size - size % vg->pe_size;
          printf ( "%s -- rounding up size to physical extend boundary %s\n",
-                  cmd, ( dummy = show_size ( size / 2, SHORT)));
+                  cmd, ( dummy = lvm_show_size ( size / 2, SHORT)));
          free ( dummy); dummy = NULL;
       }
 
@@ -399,7 +432,7 @@ int main ( int argc, char **argv) {
          printf ( "%s -- rounding %d KB to stripe boundary size ",
                   cmd, size / 2);
          size = size - size_rest + stripes * vg->pe_size;
-         printf ( "%d KB / %lu PE\n", size / 2, size / vg->pe_size);
+         printf ( "%d KB / %u PE\n", size / 2, size / vg->pe_size);
       }
    } else {
       fprintf ( stderr, "%s -- please give an l or L option\n\n", cmd);
@@ -432,7 +465,7 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- checking stripe count against "
                             "physical volume count\n", cmd);
    if ( stripes > vg->pv_cur) {
-      fprintf ( stderr, "%s -- too many stripes for %s with %lu "
+      fprintf ( stderr, "%s -- too many stripes for %s with %u "
                         "physical volume",
                 cmd, vg_name, vg->pv_cur);
       if ( vg->pv_cur > 1) fprintf ( stderr, "s");
@@ -560,13 +593,16 @@ int main ( int argc, char **argv) {
       }
       fsync ( lv_handle);
       close ( lv_handle);
+   } else {
+      printf ( "%s -- WARNING: you are not initializing data on %s\n",
+               cmd, lv_name);
    }
 
    if ( opt_v > 0) printf ( "%s -- changing lvmtab\n", cmd);
-   if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, cmd, opt_v, vg) == 0 &&
+   if ( vg_cfgbackup ( vg_name, LVMTAB_DIR, opt_v, vg) == 0 &&
         opt_A > 0) {
       printf ( "%s -- doing automatic backup of %s\n", cmd, vg_name);
-      vg_cfgbackup ( vg_name, VG_BACKUP_DIR, cmd, opt_v, vg);
+      vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
    }
 
    lvm_interrupt ();
